/*
 * Coded only for 32 bit.
 * Based on Tokugawa Ieyasu code.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __SEGMENT_CHECK_H
#define __SEGMENT_CHECK_H

#include <elf.h>
#include <stdio.h>

#define TRUE 1
#define FALSE 0

int sec_text_check( FILE* );
int sec_text_set( FILE* , int );

int sec_text_check( FILE *target )
{
	Elf32_Ehdr ELFHeader;
	Elf32_Phdr segmentHeader;
	long int position=-1;
	int i = 0;
	if ( target != NULL )
	{
		position = ftell( target );
		fseek(target, 0, SEEK_SET);		
		size_t header = fread(&ELFHeader, sizeof(Elf32_Ehdr), 1, target);
		if (header * sizeof(Elf32_Ehdr) == sizeof(Elf32_Ehdr) * 1)
		{
			do
			{
				fseek(target, ELFHeader.e_phoff + (ELFHeader.e_phentsize * i), SEEK_SET);
				size_t header = fread(&segmentHeader, ELFHeader.e_phentsize, 1, target);
				if (header * ELFHeader.e_phentsize == ELFHeader.e_phentsize * 1)
				{
					if (ELFHeader.e_entry >= segmentHeader.p_vaddr && ELFHeader.e_entry < (segmentHeader.p_vaddr + segmentHeader.p_memsz))
					{
						if ((segmentHeader.p_flags & PF_W) == PF_W)
						{
							fseek(target, position, SEEK_SET);
							return TRUE;
						} else
						{
							return FALSE;
						}
					}
				}
				i++;
			} while ( i < ELFHeader.e_phnum );
			printf("HAVE COUNTED ALL SECTIONS\n");
			fseek(target, position, SEEK_SET);
			return -1;
		}	else
		{
			fseek(target, position, SEEK_SET);
			printf("HEADER SIZE NOT MATCH \n");
			return -1;
		}
		printf("CANNOT READ HEADER \n");
		fseek(target, position, SEEK_SET);
		return -1;
	}
	printf("FILE ID = NULL\n");
	return -1;
}


int sec_text_set( FILE *target , int flag )
{
	Elf32_Ehdr ELFHeader;
	Elf32_Phdr segmentHeader;
	long int position=-1;
	int i = 0;
	if ( target != NULL )
	{
		position = ftell( target );
		fseek(target, 0, SEEK_SET);		
		size_t header = fread(&ELFHeader, sizeof(Elf32_Ehdr), 1, target);
		if (header * sizeof(Elf32_Ehdr) == sizeof(Elf32_Ehdr) * 1)
		{
			do
			{
				fseek(target, ELFHeader.e_phoff + (ELFHeader.e_phentsize * i), SEEK_SET);
				size_t header = fread(&segmentHeader, ELFHeader.e_phentsize, 1, target);
				if (header * ELFHeader.e_phentsize == ELFHeader.e_phentsize * 1)
				{
					if (ELFHeader.e_entry >= segmentHeader.p_vaddr && ELFHeader.e_entry < (segmentHeader.p_vaddr + segmentHeader.p_memsz))
					{
						if ( flag == 1)
						{
							segmentHeader.p_flags |= PF_W;
						} else
						{
							segmentHeader.p_flags =segmentHeader.p_flags & (~(segmentHeader.p_flags & (0x00000001<<1)));
						}
						fseek(target, ELFHeader.e_phoff + (ELFHeader.e_phentsize * i), SEEK_SET);
						size_t header = fwrite(&segmentHeader, ELFHeader.e_phentsize, 1, target);
						if (header * ELFHeader.e_phentsize == ELFHeader.e_phentsize * 1)
						{
							return 0;
						} else
						{
							printf("CANNOT WRITE TARGET FILE\n");
							fseek(target, position, SEEK_SET);
							return -1;
						}
					}
				}
				i++;
			} while ( i < ELFHeader.e_phnum );
			printf("HAVE COUNTED ALL SECTIONS\n");
			fseek(target, position, SEEK_SET);
			return -1;
		}	else
		{
			fseek(target, position, SEEK_SET);
			printf("HEADER SIZE NOT MATCH \n");
			return -1;
		}
		printf("CANNOT READ HEADER \n");
		fseek(target, position, SEEK_SET);
		return -1;
	}
	printf("FILE ID = NULL\n");
	return -1;
}

#endif
