/*
 * Makes writable the segment of the entry point
 * Copyright (C) 2010  Tokugawa Ieyasu
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/******************************************************************************
 * Compiling on 32 bits systems:                                              *
 *  gcc -U CONFIG_X86_64 -D CONFIG_X86_32 -O2 -o textwritable textwritable.c  *
 *                                                                            *
 * Compiling on 64 bits systems:                                              *
 *  gcc -U CONFIG_X86_32 -D CONFIG_X86_64 -O2 -o textwritable textwritable.c  *
 ******************************************************************************/


#include <elf.h>
#include <stdio.h>


#define bool _Bool
#define true 1
#define false 0


int main(int argc, char *argv[])
{
	#if defined CONFIG_X86_32
	Elf32_Ehdr ELFHeader;
	Elf32_Phdr segmentHeader;
	#elif defined CONFIG_X86_64
	Elf64_Ehdr ELFHeader;
	Elf64_Phdr segmentHeader;
	#endif

	bool writable = false;
	FILE *target = NULL;
	int i = 0;


	if (argc == 1)
	{
		printf("%s: no input files\n", argv[0]);
		return 1;
	} else
	{
		target = fopen(argv[1], "rb+");

		if (target != NULL)
		{
			fseek(target, 0, SEEK_SET);

			#if defined CONFIG_X86_32
			size_t header = fread(&ELFHeader, sizeof(Elf32_Ehdr),
					1, target);
			if (header * sizeof(Elf32_Ehdr) == sizeof(Elf32_Ehdr) * 1)
			{
			#elif defined CONFIG_X86_64
			size_t header = fread(&ELFHeader, sizeof(Elf64_Ehdr),
					1, target);
			if (header * sizeof(Elf64_Ehdr) == sizeof(Elf64_Ehdr) * 1)
			{
			#endif

				do
				{
					fseek(target, ELFHeader.e_phoff + (ELFHeader.e_phentsize * i), SEEK_SET);

					size_t header = fread(&segmentHeader, ELFHeader.e_phentsize, 1, target);
					if (header * ELFHeader.e_phentsize == ELFHeader.e_phentsize * 1)
					{
						if (ELFHeader.e_entry >= segmentHeader.p_vaddr &&
								ELFHeader.e_entry < (segmentHeader.p_vaddr + segmentHeader.p_memsz))
						{
							segmentHeader.p_flags |= PF_W;
							fseek(target, ELFHeader.e_phoff + (ELFHeader.e_phentsize * i), SEEK_SET);

							size_t header = fwrite(&segmentHeader, ELFHeader.e_phentsize, 1, target);
							if (header * ELFHeader.e_phentsize == ELFHeader.e_phentsize * 1)
							{
								writable = true;
							} else
							{
								printf("Cannot write target file\n");
								fclose(target);
								return 1;
							}
						}
					} else
					{
						printf("Cannot read target file\n");
						fclose(target);
						return 1;
					}

					i++;
				} while (!writable && i < ELFHeader.e_phnum);

				if (!writable)
				{
					printf("No segment made writable\n");
					fclose(target);
					return 1;
				}

				fclose(target);
			} else
			{
				printf("Cannot read target file\n");
				fclose(target);
				return 1;
			}
		} else
		{
			printf("Cannot open target file\n");
			return 1;
		}
	}

	printf("Everything's fine\n");
	return 0;
}

